#include "gtest/gtest.h"
#include "Effects.h"
#include "AudioFile.h"
#include <iostream>
#include <string>
#include <vector>
using namespace std;

class Testing : public ::testing::Test {

protected:
    Testing() {}
    virtual ~Testing() {
    }

    virtual void SetUp() {}
    
    virtual void TearDown() {
	cout << endl << ":::END" << endl;
    }
};
    

TEST_F(Testing, changeSampleRate) {
    cout << ":::NAME test changeSampleRate function" << endl;
    cout << ":::MAX_SCORE 5" << endl;
    cout << ":::SCORE 5" << endl;
    cout << ":::VISIBILITY visible" << endl;

    Audio empty;
    Audio slow = changeSampleRate(empty, 1000);
    cout << "Setting the default sample rate to 1000" << endl;
    EXPECT_EQ(1000, slow.getSampleRate()) << "\n:::SCORE 0";
}

TEST_F(Testing, scaleVolume) {
    cout << ":::NAME test scaleVolume" << endl;
    cout << ":::MAX_SCORE 5" << endl;
    cout << ":::SCORE 5" << endl;
    cout << ":::VISIBILITY visible" << endl;
    
    Audio volume;
    volume.setNumSamplesPerChannel(5);
    for (unsigned int index = 0; 
         index < volume.getNumSamplesPerChannel();
         index++)
	volume.samples[0][index] = (double)index;

    cout << "Testing scaleVolume with [0,1,2,3,4] and scale 0.5" << endl;
    Audio scaled = scaleVolume(volume, 0.5);

    for (unsigned int index = 0; 
         index < volume.getNumSamplesPerChannel();
         index++)
	EXPECT_DOUBLE_EQ((double)index/2.0, scaled.samples[0][index]) << "\n:::SCORE 0";
}

TEST_F(Testing, reverseAudio) {
    cout << ":::NAME test reverseAudio" << endl;
    cout << ":::MAX_SCORE 5" << endl;
    cout << ":::SCORE 5" << endl;
    cout << ":::VISIBILITY visible" << endl;
    
    Audio volume;
    volume.setNumSamplesPerChannel(5);
    for (unsigned int index = 0; 
         index < volume.getNumSamplesPerChannel();
         index++)
	volume.samples[0][index] = (double)index;

    cout << "Testing reverseAudio with [0,1,2,3,4]" << endl;
    Audio reversed = reverseAudio(volume);

    for (unsigned int index = 0; 
         index < volume.getNumSamplesPerChannel();
         index++)
	EXPECT_DOUBLE_EQ(4.0-(double)index, reversed.samples[0][index]) << "\n:::SCORE 0";
}

TEST_F(Testing, smoothAudio) {
    cout << ":::NAME test smoothAudio" << endl;
    cout << ":::MAX_SCORE 5" << endl;
    cout << ":::SCORE 5" << endl;
    cout << ":::VISIBILITY visible" << endl;
    
    Audio volume;
    volume.setNumSamplesPerChannel(5);
    for (unsigned int index = 0; 
         index < volume.getNumSamplesPerChannel();
         index++)
	volume.samples[0][index] = (double)index * index;

    cout << "Testing smoothAudio with [0,1,4,9,16]" << endl;
    Audio smooth = smoothAudio(volume);

    double result[] = {0.5, 5.0/3.0, 14.0/3.0, 29.0/3.0, 25.0/2.0}; 
    for (unsigned int index = 0; 
         index < volume.getNumSamplesPerChannel();
         index++)
	EXPECT_DOUBLE_EQ(result[index], smooth.samples[0][index]) << "\n:::SCORE 0";
}

TEST_F(Testing, echoAudio) {
    cout << ":::NAME test echoAudio" << endl;
    cout << ":::MAX_SCORE 5" << endl;
    cout << ":::SCORE 5" << endl;
    cout << ":::VISIBILITY visible" << endl;
    
    Audio volume;
    volume.setNumSamplesPerChannel(5);
    for (unsigned int index = 0; 
         index < volume.getNumSamplesPerChannel();
         index++)
	volume.samples[0][index] = (double)index;

    cout << "Testing echoAudio with [0,1,2,3,4]" << endl;
    volume.setSampleRate(1);
    Audio echo = echoAudio(volume, 1.0, 0.25);

    double result[] = {0.0, 1.0, 2.25, 3. + 0.5, 4.0 + 3*0.25};
    for (unsigned int index = 0; 
         index < volume.getNumSamplesPerChannel();
         index++)
	EXPECT_DOUBLE_EQ(result[index], echo.samples[0][index]) << "\n:::SCORE 0";
}

TEST_F(Testing, addNoise) {
    cout << ":::NAME test addNoise" << endl;
    cout << ":::MAX_SCORE 5" << endl;
    cout << ":::SCORE 5" << endl;
    cout << ":::VISIBILITY visible" << endl;
    
    Audio volume;
    volume.setNumSamplesPerChannel(100000);
    for (unsigned int index = 0; 
         index < volume.getNumSamplesPerChannel();
         index++)
	volume.samples[0][index] = (double)0.0;

    cout << "Testing echoAudio with 100,000 samples of 0.0 and noise of 0.25" << endl;
    cout << "Resulting samples should be within -0.25 to 0.25" << endl;

    Audio noise = addNoise(volume, 0.25);

    for (unsigned int index = 0; 
         index < volume.getNumSamplesPerChannel();
         index++) {
	EXPECT_LT(noise.samples[0][index], 0.251) << "\n:::SCORE 0";
	EXPECT_GT(noise.samples[0][index], -0.251) << "\n:::SCORE 0";
    }

    double max = -0.25, min = 0.25;
    for (unsigned int index = 0; 
         index < volume.getNumSamplesPerChannel();
         index++) {

	if (noise.samples[0][index] > max)
	    max = noise.samples[0][index];
	if (noise.samples[0][index] < min)
	    min = noise.samples[0][index];
   }

   cout << "There is a high probability of a sample being greater \nthan 0.2 and less than -0.2. Testing that." << endl;

   EXPECT_GT(max, 0.2) << "\n:::SCORE 0";
   EXPECT_LT(min, -0.2) << "\n:::SCORE 0";
    
}


int main(int argc, char **argv) {
  testing::InitGoogleTest(&argc, argv);
  RUN_ALL_TESTS();
  return 0;
}
